package com.javainuse.spring_boot_ai;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import com.azure.ai.openai.OpenAIClient;
import com.azure.ai.openai.OpenAIClientBuilder;
import com.azure.ai.openai.models.ChatChoice;
import com.azure.ai.openai.models.ChatCompletions;
import com.azure.ai.openai.models.ChatCompletionsOptions;
import com.azure.ai.openai.models.ChatMessage;
import com.azure.ai.openai.models.ChatRole;
import com.azure.ai.openai.models.EmbeddingItem;
import com.azure.ai.openai.models.Embeddings;
import com.azure.core.credential.AzureKeyCredential;
import com.javainuse.spring_boot_ai.elasticsearch.EmbeddingDto;
import com.javainuse.spring_boot_ai.elasticsearch.EmbeddingService;

@RestController
public class PromptController {

    @Value("${azure.openai.api.key}")
    private String azureOpenaiKey;

    @Value("${azure.openai.endpoint}")
    private String endpoint;

    @Value("${azure.openai.deployment.model.id}")
    private String deploymentOrModelId;
    
    @Value("${azure.openai.embedding.model.id}")
	private String embeddingModelId;
    
    @Autowired
	private EmbeddingService embeddingService;


    @PostMapping("/answer")
    public List<String> getMethodName(@RequestBody PromptQuestion promptQuestion) {
    	
        List<String> responseList = new ArrayList<>();
        try {
            OpenAIClient client = new OpenAIClientBuilder()
                    .endpoint(endpoint)
                    .credential(new AzureKeyCredential(azureOpenaiKey))
                    .buildClient();

            List<ChatMessage> messages = new ArrayList<>();
            messages.add(new ChatMessage(ChatRole.SYSTEM).setContent("You are an AI assistant that helps people find information."));
            messages.add(new ChatMessage(ChatRole.USER).setContent(getPrompt(promptQuestion)));

            ChatCompletionsOptions options = new ChatCompletionsOptions(messages)
                    .setTemperature(0.7)
                    .setTopP(0.95)
                    .setMaxTokens(800);

            ChatCompletions completions = client.getChatCompletions(deploymentOrModelId, options);

            for (ChatChoice choice : completions.getChoices()) {
                responseList.add(choice.getMessage().getContent().trim());
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            responseList.add("Exception Occurred");
        }
        return responseList;
    }
    
    @PostMapping("/get-embed")
	public List<Double> getEmbedding(@RequestBody EmbedData embedData) {
		try {
			OpenAIClient client = new OpenAIClientBuilder().endpoint(endpoint)
					.credential(new AzureKeyCredential(azureOpenaiKey)).buildClient();
			String input = getEmbedData(embedData);
			Embeddings embeddings = client.getEmbeddings(embeddingModelId,
					new com.azure.ai.openai.models.EmbeddingsOptions(List.of(input)));
			EmbeddingItem embeddingItem = embeddings.getData().get(0);
			List<Double> embedding = embeddingItem.getEmbedding();
			return embedding;

		} catch (Exception ex) {
			ex.printStackTrace();
			return List.of();
		}
	} 
    
    @PostMapping("/store-embed")
	public ResponseEntity<EmbeddingDto> storeEmbedding(@RequestBody EmbedData embedData) {
		try {
			String text = getEmbedData(embedData);
			List<Double> embedding = getEmbedding(embedData);

			if (embedding.isEmpty()) {
				return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
			}

			EmbeddingDto embeddingDto = new EmbeddingDto();
			embeddingDto.setId(UUID.randomUUID().toString());
			embeddingDto.setText(text);
			embeddingDto.setEmbedding(embedding);

			EmbeddingDto savedEmbedding = embeddingService.storeEmbedding(embeddingDto);
			return new ResponseEntity<>(savedEmbedding, HttpStatus.CREATED);
		} catch (Exception ex) {
			ex.printStackTrace();
			return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
    
    @PostMapping("/enhanced-answer")
	public ResponseEntity<List<String>> getEnhancedAnswer(@RequestBody PromptQuestion promptQuestion) {
		List<String> responseList = new ArrayList<>();
		try {
			// First, search for similar documents
			EmbedData data = new EmbedData();
			data.setData(getPrompt(promptQuestion));
			List<Double> queryEmbedding = getEmbedding(data);
			List<EmbeddingDto> similarDocuments = embeddingService.findSimilarDocuments(queryEmbedding);

			// Extract the most relevant contexts (top 3)
			StringBuilder context = new StringBuilder();
			int maxDocs = Math.min(3, similarDocuments.size());
			for (int i = 0; i < maxDocs; i++) {
				context.append("Context ").append(i + 1).append(": ").append(similarDocuments.get(i).getText())
						.append("\n\n");
			}

			// Create a prompt that includes this context
			String enhancedPrompt = "Based on the following contexts, please answer the user's question.\n\n"
					+ context.toString() + "User's question: " + getPrompt(promptQuestion);

			// Call OpenAI with the enhanced prompt
			OpenAIClient client = new OpenAIClientBuilder().endpoint(endpoint)
					.credential(new AzureKeyCredential(azureOpenaiKey)).buildClient();

			List<ChatMessage> messages = new ArrayList<>();
			messages.add(new ChatMessage(ChatRole.SYSTEM)
					.setContent("You are an AI assistant that helps people find information. "
							+ "Use the provided contexts to answer questions. If the contexts don't contain "
							+ "relevant information, just say you don't know."));
			messages.add(new ChatMessage(ChatRole.USER).setContent(enhancedPrompt));

			ChatCompletionsOptions options = new ChatCompletionsOptions(messages).setTemperature(0.7).setTopP(0.95)
					.setMaxTokens(800);

			ChatCompletions completions = client.getChatCompletions(deploymentOrModelId, options);
			for (ChatChoice choice : completions.getChoices()) {
				responseList.add(choice.getMessage().getContent().trim());
			}

			return new ResponseEntity<>(responseList, HttpStatus.OK);
		} catch (Exception ex) {
			ex.printStackTrace();
			responseList.add("Exception Occurred");
			return new ResponseEntity<>(responseList, HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}
    
    private String getEmbedData(EmbedData embedData) {
		String input = embedData.getData().trim();
		return input;
	}
   

    private String getPrompt(PromptQuestion promptQuestion) {
        String input = promptQuestion.getQuestion().trim();        
        return input;
    }
}
